# -*- coding: utf-8 -*-
'''
Created on 2020年11月07日
'''

import abc
import re


class FileProcessAbstractFactory(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def process_file_content(self, file_path, files_process_api):
        raise NotImplementedError("Must implement process_file_content")

    # 字符串验证
    def string_compiler(self, str, regex, re_flag=re.I):
        text = re.compile(regex, re_flag)
        if text.match(str):
            return True
        else:
            return False



from k_parse_tool.Enum import item_enum

table_title = {
    item_enum.DETAIL_PAGE_TITLE:
        [u'案件名称', u'案件名称：', u'案件名称:', u'处罚名称', u'项目名称：'],  # 标题
    item_enum.DETAIL_PAGE_DEPT:
        [u'作出处罚决定的机关名称', u'执法主体', u'许可机关:', u'处罚机关', u'处罚机关:', u'处罚机关：',
         u'登记机关', u'做出处罚的机关名称和日期', u'作出处罚的机关名称和日期', u'作出处罚机关名称和日期',
         u'作出处罚决定的部门', u'执法单位名称', u'作出处罚单位名称'],  # 发布部门
    item_enum.DETAIL_PAGE_DATE:
        [u'作出处罚的日期', u'日期', u'处罚决定日期:', u'处罚生效期：', u'处罚日期', u'处罚生效期起', u'作出决定时间',
         u'做出处罚的机关名称和日期', u'作出处罚的机关名称和日期', u'作出处罚机关名称和日期', u'处罚决定日期：',
         u'作出处理决定的机关名称和日期', u'签发日期', u'处罚决定作出日期'],  # 发布时间
    item_enum.DETAIL_PAGE_TEXT_NUMBER:
        [u'行政处罚决定书文号', u'处罚决定书文号', u'行政处罚决定文书号：', u'文书字轨', u'行政处罚决定书文号：',
         u'行政处罚决定书文号:', u'行政处理决定书文号', u'行政决定书文号', u'处罚许可决定书文号:', u'决定书号'],  # 发文字号
    item_enum.DETAIL_PAGE_PUNISHED_PARTIES:
        [u'被处罚自然人姓名、被处罚企业或其他组织名称', u'行政相对人', u'行政相对人名称：', u'行政相对人名称:',
         u'纳税人名称', u'违法企业名称或违法自然人姓名', u'被处罚人', u'被处罚对象', u'被处罚人名称', u'被处罚单位（人）名称',
         u'当事人名称'],  # 被处罚对象
    item_enum.DETAIL_PAGE_PUNISHED_PEOPLE:
        [u'负责人姓名', u'法定代表人', u'法定代表人姓名：', u'法人代表姓名:', u'法人代表姓名：',
         u'行政处罚相对人姓名', u'经营者', u'法定代表人:'],  # 被处罚人
    item_enum.DETAIL_PAGE_LAWACCORDING:
        [u'处罚依据', u'执法依据', u'处罚依据:', u'处罚依据：', u'行政处罚依据',
         u'行政处罚的种类和依据', u'行政处理的种类和依据'],  # 法律依据
    item_enum.DETAIL_PAGE_PENALTYACCORDING:
        [u'案由', u'事由', u'处罚事由', u'处罚依据'],  # 处罚依据
    item_enum.DETAIL_PAGE_PUNISHMENT_MEASURES:
        [u'处罚结果', u'处罚结果：', u'处罚结果:', u'罚款结果', u'行政处罚结果',
         u'处理结果', u'处罚内容:', u'处罚决定内容', u'处罚措施', u'处罚金额'],  # 处罚结果
}



# -*- coding: utf-8 -*-
'''
Created on 2020年11月25日
'''
import math
import re
import threading

from enum import Enum

from k_parse_tool.lib.file_process_abstract_factory import FileProcessAbstractFactory


class ExcelFileProcessAbstractFactory():
    excel_lib_type = Enum("excel_lib_type", ("Pandas", "xlrd_xlwt_xlutils", "openpyxl"))

    @staticmethod
    def get_pdf_file_process_object(pdf_lib_type):
        if ExcelFileProcessAbstractFactory.excel_lib_type.Pandas == pdf_lib_type:
            return PandasPdfFileProcess()
        else:
            pass


class PandasPdfFileProcess(FileProcessAbstractFactory):

    _instance_lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(PandasPdfFileProcess, "_instance"):
            with PandasPdfFileProcess._instance_lock:
                if not hasattr(PandasPdfFileProcess, "_instance"):
                    PandasPdfFileProcess._instance = object.__new__(cls)
        return PandasPdfFileProcess._instance

    def process_file_content(self, file_path):
        raise NotImplementedError("Must implement process_file_content")


    # 字符串验证
    def string_compiler(self, str,regex):
        text = re.compile(regex)
        if text.match(str):
            return True
        else:
            return False





__all__ = ["A", "ASCII", "B", "BESTMATCH", "D", "DEBUG", "E", "ENHANCEMATCH",
  "F", "FULLCASE", "I", "IGNORECASE", "L", "LOCALE", "M", "MULTILINE", "P",
  "POSIX", "R", "REVERSE", "S", "DOTALL", "T", "TEMPLATE", "U", "UNICODE",
  "V0", "VERSION0", "V1", "VERSION1", "W", "WORD", "X"]

# Flags.
A = ASCII = 0x80          # Assume ASCII locale.
B = BESTMATCH = 0x1000    # Best fuzzy match.
D = DEBUG = 0x200         # Print parsed pattern.
E = ENHANCEMATCH = 0x8000 # Attempt to improve the fit after finding the first
                          # fuzzy match.
F = FULLCASE = 0x4000     # Unicode full case-folding.
I = IGNORECASE = 0x2      # Ignore case.
L = LOCALE = 0x4          # Assume current 8-bit locale.
M = MULTILINE = 0x8       # Make anchors look for newline.
P = POSIX = 0x10000       # POSIX-style matching (leftmost longest).
R = REVERSE = 0x400       # Search backwards.
S = DOTALL = 0x10         # Make dot match newline.
U = UNICODE = 0x20        # Assume Unicode locale.
V0 = VERSION0 = 0x2000    # Old legacy behaviour.
V1 = VERSION1 = 0x100     # New enhanced behaviour.
W = WORD = 0x800          # Default Unicode word breaks.
X = VERBOSE = 0x40        # Ignore whitespace and comments.
T = TEMPLATE = 0x1        # Template (present because re module has it).


from k_parse_tool.lib.htmlprocess import get_table_metadata

# -*- coding: utf-8 -*-

import unittest
from unittest.mock import patch


class PdfProcessTestCase(unittest.TestCase):

    def test_extract_table_metadata(self):
        self.env = patch.dict('os.environ', {'SCRAPY_PROJECT': 'development'})
        source = """<table style="WIDTH: 1531px; BORDER-COLLAPSE: collapse" cellspacing="0" cellpadding="0" width="1531" border="0">  <colgroup>  <col style="WIDTH: 95px; mso-width-source: userset; mso-width-alt: 3040" width="95">  <col style="WIDTH: 207px; mso-width-source: userset; mso-width-alt: 6624" width="207">  <col style="WIDTH: 220px; mso-width-source: userset; mso-width-alt: 7040" width="220">  <col style="WIDTH: 295px; mso-width-source: userset; mso-width-alt: 9440" width="295">  <col style="WIDTH: 47px; mso-width-source: userset; mso-width-alt: 1504" width="47">  <col style="WIDTH: 52px; mso-width-source: userset; mso-width-alt: 1664" width="52">  <col style="WIDTH: 56px; mso-width-source: userset; mso-width-alt: 1792" width="56">  <col style="WIDTH: 59px; mso-width-source: userset; mso-width-alt: 1856" width="58">  <col style="WIDTH: 63px; mso-width-source: userset; mso-width-alt: 1984" width="62">  <col style="WIDTH: 63px; mso-width-source: userset; mso-width-alt: 2016" width="63">  <col style="WIDTH: 117px; mso-width-source: userset; mso-width-alt: 3744" width="117">  <col style="WIDTH: 259px; mso-width-source: userset; mso-width-alt: 8288" width="259">  </colgroup><tbody>    <tr style="HEIGHT: 19px" height="19">      <td class="xl65" style="BORDER-TOP: #000 1px solid; HEIGHT: 38px; BORDER-RIGHT: #000 1px solid; WIDTH: 95px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" height="38" rowspan="2" width="95"><font face="宋体"><strong>决定书号</strong></font></td>      <td class="xl73" style="BORDER-TOP: #000 1px solid; BORDER-RIGHT: #000 1px solid; WIDTH: 207px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" rowspan="2" width="207"><font face="宋体"><strong>被处罚人名称</strong></font></td>      <td class="xl73" style="BORDER-TOP: #000 1px solid; BORDER-RIGHT: #000 1px solid; WIDTH: 220px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" rowspan="2" width="220"><font face="宋体"><strong>案由</strong></font></td>      <td class="xl73" style="BORDER-TOP: #000 1px solid; BORDER-RIGHT: #000 1px solid; WIDTH: 295px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" rowspan="2" width="295"><font face="宋体"><strong>处罚依据</strong></font></td>      <td class="xl65" style="BORDER-TOP: #000 1px solid; BORDER-RIGHT: #000 1px solid; WIDTH: 339px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="338" colspan="6"><font face="宋体"><strong>处罚决定内容</strong></font></td>      <td class="xl74" style="BORDER-TOP: #000 1px solid; BORDER-RIGHT: #000 1px solid; WIDTH: 117px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" rowspan="2" width="117"><strong><font face="宋体">签发日期</font><font class="font7" face="Times New Roman"><span style="mso-spacerun: yes"> </span></font></strong></td>      <td class="xl65" style="BORDER-TOP: #000 1px solid; BORDER-RIGHT: #000 1px solid; WIDTH: 259px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" rowspan="2" width="259"><strong><font face="宋体">执法单位名称</font></strong></td>    </tr>    <tr style="HEIGHT: 19px" height="19">      <td class="xl65" style="BORDER-TOP: #000; HEIGHT: 19px; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" height="19"><strong><font face="宋体">警告</font></strong></td>      <td class="xl65" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><strong><font face="宋体">没收</font></strong></td>      <td class="xl65" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><strong><font face="宋体">罚款</font></strong></td>      <td class="xl65" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><strong><font face="宋体">吊证</font></strong></td>      <td class="xl65" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><strong><font face="宋体">停产</font></strong></td>      <td class="xl65" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><strong><font face="宋体">其他</font></strong></td>    </tr>    <tr style="HEIGHT: 76px" height="76">      <td class="xl71" style="BORDER-TOP: #000; HEIGHT: 76px; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" height="76"><font face="宋体">2120162065</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 207px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="207"><font face="宋体">沈建芳（上海闵行虹桥美美美发厅的经营者）</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 220px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="220"><font face="宋体">未按照规定对公共场所的空气、微小气候、水质、采光、照明、噪声、顾客用品用具等进行卫生检测</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 295px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="295"><font face="宋体">《公共场所卫生管理条例实施细则》第三十六条第（一）项</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1100</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl70" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">2016-5-4</font></td>      <td class="xl69" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">上海市闵行区卫生和计划生育委员会</font></td>    </tr>    <tr style="HEIGHT: 76px" height="76">      <td class="xl71" style="BORDER-TOP: #000; HEIGHT: 76px; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" height="76"><font face="宋体">2120162064</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 207px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="207"><font face="宋体">上海相昶贸易有限公司</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 220px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="220"><font face="宋体">未按照规定对公共场所的空气、微小气候、水质、采光、照明、噪声、顾客用品用具等进行卫生检测</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 295px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="295"><font face="宋体">《公共场所卫生管理条例实施细则》第三十六条第（一）项</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1100</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl70" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">2016-5-4</font></td>      <td class="xl69" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">上海市闵行区卫生和计划生育委员会</font></td>    </tr>    <tr style="HEIGHT: 76px" height="76">      <td class="xl71" style="BORDER-TOP: #000; HEIGHT: 76px; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" height="76"><font face="宋体">2120162066</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 207px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="207"><font face="宋体">郭祥娣（上海市闵行区浦江镇清星理发店的经营者）</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 220px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="220"><font face="宋体">未按照规定对公共场所的空气、微小气候、水质、采光、照明、噪声、顾客用品用具等进行卫生检测</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 295px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="295"><font face="宋体">《公共场所卫生管理条例实施细则》第三十六条第（一）项</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1100</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl70" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">2016-5-4</font></td>      <td class="xl69" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">上海市闵行区卫生和计划生育委员会</font></td>    </tr>    <tr style="HEIGHT: 76px" height="76">      <td class="xl71" style="BORDER-TOP: #000; HEIGHT: 76px; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" height="76"><font face="宋体">2120162063</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 207px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="207"><font face="宋体">魏学宽（上海市闵行区欣品空间美发厅的经营者）</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 220px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="220"><font face="宋体">未按照规定对公共场所的空气、微小气候、水质、采光、照明、噪声、顾客用品用具等进行卫生检测</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 295px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="295"><font face="宋体">《公共场所卫生管理条例实施细则》第三十六条第（一）项</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1200</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl70" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">2016-5-4</font></td>      <td class="xl69" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">上海市闵行区卫生和计划生育委员会</font></td>    </tr>    <tr style="HEIGHT: 76px" height="76">      <td class="xl71" style="BORDER-TOP: #000; HEIGHT: 76px; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000 1px solid; BACKGROUND-COLOR: transparent" height="76"><font face="宋体">2120162067</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 207px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="207"><font face="宋体">上海凯姆美容美发有限公司</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 220px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="220"><font face="宋体">未按照规定对公共场所的空气、微小气候、水质、采光、照明、噪声、顾客用品用具等进行卫生检测</font></td>      <td class="xl72" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; WIDTH: 295px; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent" width="295"><font face="宋体">《公共场所卫生管理条例实施细则》第三十六条第（一）项</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl71" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">1100</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl67" style="BORDER-TOP: #000; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">　</font></td>      <td class="xl70" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">2016-5-4</font></td>      <td class="xl69" style="BORDER-TOP: #ece9d8; BORDER-RIGHT: #000 1px solid; BORDER-BOTTOM: #000 1px solid; BORDER-LEFT: #000; BACKGROUND-COLOR: transparent"><font face="宋体">上海市闵行区卫生和计划生育委员会</font></td>    </tr>  </tbody>  </table>"""
        res = get_table_metadata(source)
        print(res)


if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
'''
Created on 2020年11月25日
'''
import math
import re
import threading

import pdfplumber
from enum import Enum

from k_parse_tool.lib.file_process_abstract_factory import FileProcessAbstractFactory


class PdfFileProcessAbstractFactory():
    pdf_lib_type = Enum("pdf_lib_type", ("pdfplumber", "pdfminer"))

    @staticmethod
    def get_pdf_file_process_object(pdf_lib_type):
        if PdfFileProcessAbstractFactory.pdf_lib_type.pdfplumber == pdf_lib_type:
            return PdfplumbePdfFileProcess()
        else:
            pass

    @staticmethod
    def string_compiler(str_input, regex, re_flag=re.I):
        text = re.compile(regex, re_flag)
        if text.match(str_input):
            return True
        else:
            return False

    @staticmethod
    def pdf_file_content_format(file_content):
        # 去除空行,去除行中仅有数字,行中仅有特殊字符
        if file_content is None or len(file_content.strip()) == 0: return ""
        file_content_line_array = file_content.split('\n')
        list = []
        count = 0
        result = ''
        for line in range(len(file_content_line_array)):
            line_value = file_content_line_array[line]
            list.append(line_value)
            count += len(line_value.strip())
        avg = math.ceil(count / len(list))
        for i in range(len(list)):
            i_value = ''.join(list[i].split())
            if PdfFileProcessAbstractFactory.string_compiler(i_value, r"^[\s—]*[\d\s]*?[\s—]*$"): continue
            if PdfFileProcessAbstractFactory.string_compiler(i_value, r"^\s*第[\d\S]{0,3}页\s*$"): continue
            if PdfFileProcessAbstractFactory.string_compiler(i_value, r"^\s*第\d*页\s*，?\s*共\d*页\s*$"): continue
            if PdfFileProcessAbstractFactory.string_compiler(i_value,
                                    r"^\s*本?文书一式\s*.?\s*份[。，：]\s*?((其中)?.{0,2}\s*份[（）\s\u4e00-\u9fa5]{1,20}[，。]?\s*){1,}$"): continue
            sep = ''
            if len(i_value) < avg or PdfFileProcessAbstractFactory.string_compiler(i_value, r"^[\s\S]*?([：；。:;.）]+|[\da-z]{5,})$", re.I):
                sep = '\n'
            result = f'{result}{i_value}{sep}'
        return result


class PdfplumbePdfFileProcess(FileProcessAbstractFactory):
    _instance_lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(PdfplumbePdfFileProcess, "_instance"):
            with PdfplumbePdfFileProcess._instance_lock:
                if not hasattr(PdfplumbePdfFileProcess, "_instance"):
                    PdfplumbePdfFileProcess._instance = object.__new__(cls)
        return PdfplumbePdfFileProcess._instance

    def process_file_content(self, file_path, files_process_api):
        content = ''
        with pdfplumber.open(file_path) as pdf:
            for page in pdf.pages:
                page_content = page.extract_text()
                if page_content is not None and len(page_content) > 0:
                    content = f'{content}{page_content.strip()}\n'
                # page.extract_tables(table_settings={})  # 提取表格
                # print(content)
        return content



from k_parse_tool.lib.word_file_process import WordFileProcessAbstractFactory

from k_parse_tool.lib.pdf_file_process import PdfFileProcessAbstractFactory


class FileProcessFactory:
    @staticmethod
    def get_file_content(file_path, files_process_api, file_lib_type=None):
        if file_path is None or len(file_path.strip()) == 0: return ''
        lower_file_path = file_path.lower()
        if lower_file_path.endswith('.pdf') or lower_file_path.endswith('.pdfx'):
            pdf_extract = PdfFileProcessAbstractFactory.get_pdf_file_process_object(
                PdfFileProcessAbstractFactory.pdf_lib_type.pdfplumber)
            result = pdf_extract.process_file_content(file_path, files_process_api)
            result = PdfFileProcessAbstractFactory.pdf_file_content_format(result)
            return result
        elif lower_file_path.endswith('.doc') or lower_file_path.endswith('.docx'):
            word_extract = WordFileProcessAbstractFactory.get_word_file_process_object(file_path.split('.')[-1])
            result = word_extract.process_file_content(file_path, files_process_api)
            return result
        elif lower_file_path.endswith('.xls') or lower_file_path.endswith('.xlsx'):
            pass
        else:
            return ''

    @staticmethod
    def convent_file_to_html(file_path):
        raise NotImplementedError("Must implement process_file_content")



# -*- coding: utf-8 -*-
'''
Created on 2020年11月05日
'''
import re
import html

import numpy
import pandas as pd
from bs4 import BeautifulSoup

from k_parse_tool.htmlTemplete.HtmlTableHead import table_title


class HtmlProcess:
    @staticmethod
    def replace_html(html_source):
        cleaned = html.unescape(html_source.strip())

        cleaned = re.sub(r"&amp;", "&", cleaned)
        cleaned = re.sub(r"&lt;", "<", cleaned)
        cleaned = re.sub(r"&gt;", ">", cleaned)
        cleaned = re.sub(r"&middot;", "·", cleaned)
        cleaned = re.sub(r"&shy;", "", cleaned)
        cleaned = re.sub(r"&#160;", " ", cleaned)
        cleaned = re.sub(r"", "", cleaned)
        cleaned = re.sub(r"[ ]", " ", cleaned)
        # 替换html中的<p>
        cleaned = re.sub(r"\s*<p[^>]*?>", "\r\n", cleaned)
        cleaned = re.sub(r"</p>\s*", "\r\n", cleaned)
        cleaned = re.sub(r"\（(相关资料:)\w*\）", "\r\n", cleaned)
        cleaned = re.sub(r"<[/]?center>", "\r\n", cleaned)
        cleaned = re.sub(r"<br\s*?[/]?>", "\r\n", cleaned)
        cleaned = re.sub(r"\[接上页\]", "", cleaned)

        cleaned = HtmlProcess.clean_html(cleaned)
        return cleaned.strip()

    @staticmethod
    def clean_html(html_source):  # 利用nltk的clean_html()函数将html文件解析为text文件
        # First we remove inline JavaScript/CSS:
        cleaned = re.sub(r"(?is)<(script|style).*?>.*?(</\1>)", "", html_source.strip())
        # Then we remove html comments. This has to be done before removing regular
        # tags since comments can contain '>' characters.
        cleaned = re.sub(r"(?s)<!--(.*?)-->[\n]?", "", cleaned)
        # Next we can remove the remaining tags:
        cleaned = re.sub(r"(?s)<.*?>", " ", cleaned)
        # Finally, we deal with whitespace
        cleaned = re.sub(r"&nbsp;", " ", cleaned)
        cleaned = re.sub(r"  ", " ", cleaned)
        cleaned = re.sub(r" ", "", cleaned)
        return cleaned.strip()


def is_chinese(text):
    zh_pattern = re.compile(u'[\u4e00-\u9fa5]+')
    for ch in text:
        match = zh_pattern.search(ch)
        if (match):
            return True
    return False


def extract_metadata_by_pandas(url_source, header=0, row_index=0, table_title={}):
    soup = BeautifulSoup(url_source, 'lxml')
    table = pd.read_html(soup.prettify(), header=header, na_values='', keep_default_na=True)[0]
    pd.set_option('display.max_rows', None)
    table.columns = [''.join(x.split()) for x in table.columns]
    for x in table.columns:
        if 'float64' == table[x].dtypes:
            table[x].replace(numpy.nan, 0, inplace=True)
            table[x] = table[x].astype('int')
            table[x] = table[x].astype('str')
        elif 'int64' == table[x].dtypes:
            table[x] = table[x].astype('str')
    #table = table.astype(str)
    #print(table.dtypes)
    rows = table.iloc[row_index, :]  # table.iloc[0,:]等价于table.loc[0]
    result = {}
    for columns, columns_title_list in table_title.items():
        for columns_title in columns_title_list:
            try:
                if columns_title in rows:
                    columns_value = rows[columns_title]
                    result[columns] = columns_value.strip()
                    break
            except Exception as e:
                print(e)
    return result


def acquire_row_index(source_code):
    soups = BeautifulSoup(source_code, "lxml")
    soups.prettify()
    head_row_index = 0
    data_row_index = 0
    tr_list = soups.find_all('tr', )
    try:
        for tr_index, tr_tag in enumerate(tr_list):
            row_tr = tr_tag.find_all(re.compile(r"t[dh]"))
            # print(tr_tag.text.strip())
            if len(row_tr) >= 5 and is_chinese(tr_tag.text.strip()):  # len(row_tr) == tr_child_count
                head_row_index = tr_index
                break
            elif tr_tag.name == 'th':
                head_row_index = tr_index
                break
        head_td_count = 0
        for tr_index, tr_tag in enumerate(tr_list):
            row_tr = tr_tag.find_all(re.compile("t[dh]"))
            if len(row_tr) <= 0 : continue
            if tr_index == head_row_index:
                head_td_count = len(row_tr)
                continue
            tr_number = row_tr[0]
            if '1' == tr_number.text.strip():
                data_row_index = tr_index
                break
            elif len(row_tr) >= head_td_count:
                data_row_index = tr_index
                break
    except Exception:
        pass
    return {'head_index': head_row_index, 'data_index': data_row_index}


def get_table_metadata(html_source):
    result = {}
    try:
        index_dict = acquire_row_index(html_source)
        head_index = index_dict['head_index']
        data_index = index_dict['data_index']
        data_index = 0 if data_index == 0 else data_index - head_index
        data_index = 0 if data_index == 0 else data_index - 1
        result = extract_metadata_by_pandas(html_source, header=head_index, row_index=data_index, table_title=table_title)
    except Exception as msg:
        print(f' extract table metadata error : {msg}')
    return result


# Define here the models for your scraped items
#
# See documentation in:
# https://docs.scrapy.org/en/latest/topics/items.html

import scrapy


class WebincrementcrawlerItem(scrapy.Item):
    # define the fields for your item here like:
    # name = scrapy.Field()
    navig_url = scrapy.Field()  # 导航页url
    navig_title = scrapy.Field()  # 列表页标题
    navig_date = scrapy.Field()  # 列表页时间
    navig_dept = scrapy.Field()  # 列表页部门
    navig_text_number = scrapy.Field()  # 列表页发文字号
    navig_punished_parties = scrapy.Field()  # 列表页处罚对象
    navig_punished_people = scrapy.Field()  # 列表页处罚对象(负责人)
    navig_lawAccording = scrapy.Field()  # 列表页法律依据
    navig_penaltyAccording = scrapy.Field()  # 列表页处罚依据
    navig_punishment_measures = scrapy.Field()  # 列表页处罚结果

    detail_page_url = scrapy.Field()  # 详情页url
    detail_page_title = scrapy.Field()  # 详情页标题
    detail_page_dept = scrapy.Field()  # 详情页发文部门
    detail_page_date = scrapy.Field()  # 详情页发文时间
    detail_page_text_number = scrapy.Field()  # 详情页发文字号
    detail_page_punished_parties = scrapy.Field()  # 详情页处罚对象
    detail_page_punished_people = scrapy.Field()  # 详情页处罚对象(负责人)
    detail_page_lawAccording = scrapy.Field()  # 详情页法律依据
    detail_page_penaltyAccording = scrapy.Field()  # 详情页处罚依据
    detail_page_punishment_measures = scrapy.Field()  # 详情页处罚结果
    detail_page_body_code = scrapy.Field()  # 详情页正文
    detail_page_body_is_table = scrapy.Field()  # 详情页正文是否是表格,1、True是表格,2、False是文本
    detail_page_source_code = scrapy.Field()  # 详情页源码

    file_urls = scrapy.Field()  # 附件url列表
    file_names = scrapy.Field()  # 附件展示名称
    files = scrapy.Field()  # 附件下载路径、下载的url和文件的校验码
    files_body_is_down = scrapy.Field()  # 附件处理方式,1、下载,2、解析内容,3、解析内容并提供下载
    files_body_is_table = scrapy.Field()  # 附件是否是表格

    files_title = scrapy.Field()
    files_date = scrapy.Field()
    files_dept = scrapy.Field()
    files_text_number = scrapy.Field()
    files_punished_parties = scrapy.Field()
    files_punished_people = scrapy.Field()
    files_lawAccording = scrapy.Field()
    files_penaltyAccording = scrapy.Field()
    files_punishment_measures = scrapy.Field()
    files_body_code = scrapy.Field()

    task_code = scrapy.Field()  # 任务码
    collection_date = scrapy.Field()  # 采集时间
    admin_penalty_case_ID = scrapy.Field  # 行政处罚id
    is_allow_issued = scrapy.Field  # 是否允许发布



import re
import regex

from k_parse_tool.Enum import regex_flags, item_enum

from k_parse_tool.Enum.fix_const_enum import FixConstEnum
from prettytable import basestring

from k_parse_tool.lib.column_common_reg import LawAccordingReg, AdminPenaltyDateReg, AdminPenaltyDeptReg, \
    AdminPenaltyNumberReg, AdminPenaltyPunishedPartiesReg, AdminPenaltyPunishedPeopleReg, \
    AdminPenaltyPunishedMeasuresReg


class CommonExtractData:

    @staticmethod
    def extract_data_by_common_reg(item, resource, reg_dict):

        reg_dict = CommonExtractData.add_data_to_item_by_common_reg_from_resource(reg_dict)

        CommonExtractData.extract_data_from_dict_and_resource(reg_dict, resource, item)

    @staticmethod
    def add_data_to_item_by_common_reg_from_resource(reg_dict={}):
        # 发布部门
        if item_enum.DETAIL_PAGE_DEPT not in reg_dict.keys():
            reg_dict[item_enum.DETAIL_PAGE_DEPT] = AdminPenaltyDeptReg.getCommonDeptReg()
        # 处罚日期
        if item_enum.DETAIL_PAGE_DATE not in reg_dict.keys():
            reg_dict[item_enum.DETAIL_PAGE_DATE] = AdminPenaltyDateReg.getCommonAdminPenaltyDateReg()
        # 发文字号
        if item_enum.DETAIL_PAGE_TEXT_NUMBER not in reg_dict.keys():
            reg_dict[item_enum.DETAIL_PAGE_TEXT_NUMBER] = AdminPenaltyNumberReg.getCommonNumberReg()

        # 法律依据
        if item_enum.DETAIL_PAGE_LAWACCORDING not in reg_dict.keys():
            reg_dict[item_enum.DETAIL_PAGE_LAWACCORDING] = LawAccordingReg.getCommonLawAccordingReg()
        # 处罚对象(公司)
        if item_enum.DETAIL_PAGE_PUNISHED_PARTIES not in reg_dict.keys():
            reg_dict[
                item_enum.DETAIL_PAGE_PUNISHED_PARTIES] = AdminPenaltyPunishedPartiesReg.getCommonPunishedPartiesReg()
        # 处罚负责人
        if item_enum.DETAIL_PAGE_PUNISHED_PEOPLE not in reg_dict.keys():
            reg_dict[item_enum.DETAIL_PAGE_PUNISHED_PEOPLE] = AdminPenaltyPunishedPeopleReg.getCommonPunishedPeopleReg()
        # 处罚结果
        if item_enum.DETAIL_PAGE_PUNISHMENT_MEASURES not in reg_dict.keys():
            reg_dict[
                item_enum.DETAIL_PAGE_PUNISHMENT_MEASURES] = AdminPenaltyPunishedMeasuresReg.getCommonPunishedMeasuresReg()
        return reg_dict

    @staticmethod
    # reg_dict  的key 是item的属性名称,  value  是提取该属性的正则表达式
    def extract_data_from_dict_and_resource(reg_dict, resource, item):
        if (len(reg_dict) > 0):
            for key, value in reg_dict.items():
                if key not in item.keys():
                    end_value = ""
                    # 法律依据提取多条
                    if key == item_enum.DETAIL_PAGE_LAWACCORDING:
                        list_value = regex.findall(value, resource)
                        end_value = "\r\n".join(list_value) if len(list_value) > 0 else end_value
                        end_value = CommonExtractData.get_law_according(end_value)
                    else:
                        reg_result = regex.search(value, resource)
                        if (reg_result):
                            end_value = reg_result.group()

                    item[key] = end_value

    @staticmethod
    def join_post_params_and_url(url, post_params):
        is_str = isinstance(post_params, basestring)
        join_str = ''
        if is_str:
            join_str = url + '<POST>' + post_params + '</POST>'
        else:
            str_list = []
            for key, value in post_params.items():
                str_list.append(str(key) + '=' + str(value))
            join_str = "&".join(str_list)
            join_str = url + '<POST>' + join_str + '</POST>'
        return join_str

    @staticmethod
    def get_content_by_reg(resource, reg, search_full=False, flags=regex_flags.I, timeout=10, ignore_unused=True,
                           pos=None,
                           endpos=None):
        '''
        根据正则提取文本
        :param resource:匹配时使用的文本
        :param reg:正则表达式
        :param search_full:获取所有匹配值,默认False(仅匹配第一个)
        :param flags:可选标志修饰符,默认regex_flags.I
        :param timeout:正则匹配超时时间,默认10s
        :param ignore_unused:如果有任何未使用的关键字参数,ValueError将被抛出,默认忽略错误
        :param pos:文本中正则表达式开始搜索的位置
        :param endpos:文本中正则表达式结束搜索的位置
        :return:
        '''
        result_dict = dict()
        result_dict[FixConstEnum.REG_CONTENT] = ""
        if search_full:
            list_value = regex.findall(reg, resource, flags=flags, timeout=timeout, ignore_unused=ignore_unused,
                                       pos=pos, endpos=endpos)
            result_dict[FixConstEnum.REG_CONTENT] = list_value
        else:
            reg_result = regex.search(reg, resource, flags=flags, timeout=timeout, ignore_unused=ignore_unused, pos=pos,
                                      endpos=endpos)
            if (reg_result):
                end_value = reg_result.group()
                result_dict[FixConstEnum.REG_CONTENT] = end_value
        return result_dict

    @staticmethod
    def get_law_according(resource):
        law_reg = LawAccordingReg.getCommonLawAccordingReg()
        list_value = regex.findall(law_reg, resource)
        end_value = "\r\n".join(list_value) if len(list_value) > 0 else ''
        return end_value



#!/usr/bin/env python3
# _*_ coding: utf-8 _*_
__author__ = "kuser"

from setuptools import setup, find_packages

# with open("README.md", "r") as fh:
# long_description = fh.read()

setup(name='k_parse_tool',
      version='0.0.4',  # 版本号
      description='parse and extract data from HTML',  # 包的介绍
      author='kuser',  # 作者 就是我啦～
      author_email='xjliu@kland.com.cn',  # 你的邮箱
      url='http://xjliu123@192.168.1.200/v3_personal/k-parse-tools.git',  # 项目地址，一般的填git地址 也可以是任意可用的url 不过我喜欢使用 git
      packages=find_packages(),  # Python导入包的列表，我们使用find_packages() 来自动收集
      long_description='parse and extract data from HTML',  # 项目的描述 一般是 string 上文中定义了它
      long_description_content_type="text/markdown",  # 描述文档 README 的格式 一般我喜欢MD. 也可以是你喜欢的其他格式 支不支持我就不知道了～ 估计HTML 是支持的
      license="GPLv3",  # 开源协议
      # 这 需要去官网查，在下边提供了许可证连接 或者 你可以直接把我的粘贴走
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
          "Operating System :: OS Independent"],

      python_requires='>=3.6.3',  # Python 的版本约束
      # 其他依赖的约束
      install_requires=[
          "pdfplumber>=0.5.24",
          "python-docx>=0.8.10",
          "regex>=2020.11.13",
          "pandas>=1.1.5",
          "beautifulsoup4>=4.9.3",
          "numpy>=1.18.0",
      ]
      )



class TableTemplete():

    #dc 是一个字典类型,key是jsonObject中的key, value是该key所对应的页面上的名称
    #例如 jsonObject={documentNum:"广公机场（二派）行罚决[2020]40"}
    # dc = {"documentNum":"发文字号"}
    @staticmethod
    def getTableHtmlByDictionaryAndJson(dc,jsonObject,type):
        if(type ==1):
            return TableTemplete.crosswiseTableTemplete(dc,jsonObject)
        else:
            return TableTemplete.columnTableTemplete(dc,jsonObject)

    #table的标题是纵向显示
    @staticmethod
    def columnTableTemplete(dc,jsonObject):
        string_list  = []
        if(len(dc)> 0):
            string_list.append("<table border=\"1\">")
            for key,value in dc.items():
                json_value = jsonObject[key]
                td_value =  '' if json_value == None else json_value
                string_list.append("<tr><td>"+str(value)+": </td><td>"+str(td_value)+"</td></tr>")
            string_list.append("</table>")

        return "".join(string_list)

    # table的标题是横向显示
    @staticmethod
    def crosswiseTableTemplete(dc,jsonObject):
        table_list = []
        value_list =[]
        if (len(dc) > 0):
            table_list.append("<table border=\"1\">")
            #拼接表头
            table_list.append("<tr>");
            for key, value in dc.items():
                table_list.append("<td>"+str(value)+"</td>")
                value_list.append(key);
            table_list.append("</tr>");
            #拼接内容
            table_list.append("<tr>");
            for value in value_list:
                json_value = jsonObject[value]
                td_value = '' if json_value == None else json_value
                table_list.append("<td>" + str(td_value) + "</td>")
            table_list.append("</tr>");
            table_list.append("</table>")
        return "".join(table_list)

    # table的标题是纵向显示
    @staticmethod
    def columnTableTempleteByDictionary(dc):
        string_list = []
        if (len(dc) > 0):
            string_list.append("<table border=\"1\" style='width:70%'>")
            for key, value in dc.items():
                td_value = '' if value == None else value
                string_list.append("<tr><td>" + str(key) + ": </td><td>" + str(td_value) + "</td></tr>")
            string_list.append("</table>")

        return "".join(string_list)
    # table的标题是横向显示
    @staticmethod
    def crosswiseTableTempleteByDictionary(dc):
            table_list = []
            value_list = []
            if (len(dc) > 0):
                table_list.append("<table border=\"1\">")
                # 拼接表头
                table_list.append("<tr>");
                for key, value in dc.items():
                    table_list.append("<td>" + str(key) + "</td>")
                    value_list.append(value);
                table_list.append("</tr>");
                # 拼接内容
                table_list.append("<tr>");
                for value in value_list:
                    td_value = '' if value == None else value
                    table_list.append("<td>" + str(td_value) + "</td>")
                table_list.append("</tr>");
                table_list.append("</table>")
            return "".join(table_list)



NAVIG_URL = "navig_url"
NAVIG_TITLE = "navig_title"
NAVIG_DATE = "navig_date"
NAVIG_DEPT = "navig_dept"
NAVIG_TEXT_NUMBER = "navig_text_number"
NAVIG_PUNISHED_PARTIES = "navig_punished_parties"
NAVIG_PUNISHED_PEOPLE = "navig_punished_people"
NAVIG_LAWACCORDING = "navig_lawAccording"
NAVIG_PENALTYACCORDING = "navig_penaltyAccording"
NAVIG_PUNISHMENT_MEASURES = "navig_punishment_measures"

DETAIL_PAGE_URL = "detail_page_url"
DETAIL_PAGE_TITLE = "detail_page_title"
DETAIL_PAGE_DEPT = "detail_page_dept"
DETAIL_PAGE_DATE = "detail_page_date"
DETAIL_PAGE_TEXT_NUMBER = "detail_page_text_number"
DETAIL_PAGE_PUNISHED_PARTIES = "detail_page_punished_parties"
DETAIL_PAGE_PUNISHED_PEOPLE = "detail_page_punished_people"
DETAIL_PAGE_LAWACCORDING = "detail_page_lawAccording"
DETAIL_PAGE_PENALTYACCORDING = "detail_page_penaltyAccording"
DETAIL_PAGE_PUNISHMENT_MEASURES = "detail_page_punishment_measures"
DETAIL_PAGE_BODY_CODE = "detail_page_body_code"
DETAIL_PAGE_BODY_IS_TABLE = "detail_page_body_is_table"
DETAIL_PAGE_SOURCE_CODE = "detail_page_source_code"

FILE_URLS = "file_urls"
FILE_NAMES = "file_names"
FILES = "files"
FILES_BODY_IS_DOWN = "files_body_is_down"
FILES_BODY_IS_TABLE = "files_body_is_table"
FILES_TITLE = "files_title"
FILES_DATE = "files_date"
FILES_DEPT = "files_dept"
FILES_TEXT_NUMBER = "files_text_number"
FILES_PUNISHED_PARTIES = "files_punished_parties"
FILES_PUNISHED_PEOPLE = "files_punished_people"
FILES_LAWACCORDING = "files_lawAccording"
FILES_PENALTYACCORDING = "files_penaltyAccording"
FILES_PUNISHMENT_MEASURES = "files_punishment_measures"
FILES_BODY_CODE = "files_body_code"

TASK_CODE = "task_code"
COLLECTION_DATE = "collection_date"
ADMIN_PENALTY_CASE_ID = "admin_penalty_case_ID"
IS_ALLOW_ISSUED = "is_allow_issued"


# -*- coding: utf-8 -*-
'''
Created on 2020年11月25日
'''
import base64
import os
import math
import re
import threading

import platform

import requests
from k_parse_tool.lib.file_process_abstract_factory import FileProcessAbstractFactory

is_win_env = False
if platform.system().lower() == 'windows':
    is_win_env = True
    import win32com
    import win32com.client
    import pythoncom
# elif platform.system().lower() == 'linux':
#     import subprocess

from docx import Document



class WordFileProcessAbstractFactory():

    @staticmethod
    def get_word_file_process_object(word_file_type):
        lower_word_file_type = word_file_type.lower()
        if lower_word_file_type == 'docx':
            return PyDocXFileProcess()
        else:
            return PyDocFileProcess()


class PyDocXFileProcess(FileProcessAbstractFactory):
    _instance_lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(PyDocXFileProcess, "_instance"):
            with PyDocXFileProcess._instance_lock:
                if not hasattr(PyDocXFileProcess, "_instance"):
                    PyDocXFileProcess._instance = object.__new__(cls)
        return PyDocXFileProcess._instance

    def process_file_content(self, file_path, files_process_api):
        result = ''
        try:
            docx = Document(file_path)
            for para in docx.paragraphs:
                line = para.text
                result = f'{result}\n{line}'
        except:
            pass
        return result


class PyDocFileProcess(FileProcessAbstractFactory):
    _instance_lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(PyDocFileProcess, "_instance"):
            with PyDocFileProcess._instance_lock:
                if not hasattr(PyDocFileProcess, "_instance"):
                    PyDocFileProcess._instance = object.__new__(cls)
        return PyDocFileProcess._instance

    def process_file_content(self, file_path, files_process_api):
        if is_win_env:
            # win环境
            return PyDocFileProcess.get_word_content_by_win32(file_path)
        else:
            return PyDocFileProcess.get_word_content_by_api(file_path, files_process_api)

    @staticmethod
    def get_word_content_by_win32(file_path):
        result = ''
        try:
            pythoncom.CoInitialize()
            ms_word =  win32com.client.Dispatch("Word.Application")
            ms_word.Visible = 0  # 后台运行
            ms_word.DisplayAlerts = 0  # 不显示，不警告
            doc = ms_word.Documents.Open(file_path)
            for i in doc.Paragraphs:
                line = i.Range.Text
                result = f'{result}\n{line}'
            doc.Close()
            ms_word.Quit()
        except:
            pass
        return result

    # # 通过antiword提取到的word格式不对，暂时弃用
    # @staticmethod
    # def get_word_content_by_antiword(file_path):
    #     result = ''
    #     try:
    #         output = subprocess.check_output(["antiword", file_path])
    #         # 解码
    #         result = output.decode('utf8')
    #     except:
    #         pass
    #     return result

    @staticmethod
    def get_word_content_by_api(file_path, files_process_api):
        result = ''
        try:
            with open(file_path, 'rb') as f:
                base64_data = base64.b64encode(f.read())
                s = base64_data.decode()
                print('word:word/doc;base64,%s' % s)
                data = {
                    'fileCode': str(s),
                    'fileSuffix': "doc"
                }
                response = requests.post(files_process_api, data=data)
                result = response.text
        except:
            pass
        return result

    def word_file_content_format(self, file_content):
        # 去除空行,去除行中仅有数字,行中仅有特殊字符
        if file_content is None or len(file_content.strip()) == 0: return ""
        file_content_line_array = file_content.split('\n')
        list = []
        count = 0
        result = ''
        for line in range(len(file_content_line_array)):
            line_value = file_content_line_array[line]
            if self.string_compiler(line_value, r"^\s*[\d\s]*?\s*$"): continue
            if self.string_compiler(line_value, r"^\s*第[\d\S]{0,3}页\s*$"): continue
            if self.string_compiler(line_value, r"^\s*第\d*页\s*，?\s*共\d*页\s*$"): continue
            if self.string_compiler(line_value,
                                    r"^\s*本?文书一式\s*.?\s*份[。，：]\s*?((其中)?.{0,2}\s*份[（）\s\u4e00-\u9fa5]{1,20}[，。]?\s*){1,}$"): continue
            list.append(line_value)
            count += len(line_value)
        avg = math.ceil(count / len(list))
        for i in range(len(list)):
            i_value = list[i].strip()
            sep = ''
            if len(i_value) < avg or self.string_compiler(i_value, r"^[\s\S]*?[：；。:;.）]$"):
                sep = '\n'
            result = f'{result}{i_value}{sep}'
        return result
